/* Autor: Sergio Blanco Cuaresma
 * Data: Agost 2002
 * Descripci: TAD Gestor d'impressores
 */

public class Gestor {
  private int MAX_IMPR;   // Nombre d'impressores
  private Impressora[] printer;
  private Llista users;    // Usuaris del sistema, es far us del tipus Llista
  private int usuaris;    // Nombre d'usuaris mxim


  // Inicialitzacions.
  //  NOTA: Les impressores estaran nomerades de 0 a (maximpr-1)
  Gestor(int maxuser, int maximpr, int mem) throws NoMemoria {
    printer = new Impressora[maximpr];

    for (int i=0;i<maximpr;i++){
      printer[i] = new Impressora(mem);
    }
    users = new Llista();
    MAX_IMPR = maximpr;
    usuaris = maxuser;
  }

  // Mostrar per pantalla el contingut de les cues de cada impressora
  public void imprimir() {
    System.out.println(">>>>>> Gestor: " + MAX_IMPR);
    for (int i=0;i<MAX_IMPR;i++){
      System.out.println("- Impressora: " + i);
      printer[i].imprimir_info();
    }
    System.out.println(">>>>>>>>>>>>>>>>>");
  }
  
  
  public void afegir_usuari(String userid) throws NoMemoria, NoMoreUsers {
    Usuari u = new Usuari(userid);
    
    if (usuaris == 0) {
      throw new NoMoreUsers();
    }
    users.insertar(u);
    usuaris--;
  }

  
  public void afegir_treball(int pr, Treball t) throws PDIdreta, NoMemoria, NoImpressora, TooBig, NoUser {
    // Existeix la impressora?
    if (pr>=MAX_IMPR){
      throw new NoImpressora();
    }

    // Busquem l'usuari propietari del treball
    users.principi();
    while ((!users.finalLlista()) && (!((Usuari)(users.actual())).id().equals(t.usuari()))){
      users.avanzar();
    }
    if (((Usuari)(users.actual())).id().equals(t.usuari())) {  // Trobat
      ((Usuari)(users.actual())).treball_enviat(t);   // Afegim a la seva llista el treball que ha enviat
    } else {
      throw new NoUser();
    }
    printer[pr].afegir(t);    // I afegim a la cua d'impressi
    
  }

  public Treball primer_treball(int pr) throws NoImpressora, NoTreball, CuaBuida {
    Treball t;
    
    // Existeix la impressora?
    if (pr>=MAX_IMPR){
      throw new NoImpressora();
    }
    t = printer[pr].primer_treball();
    if (t==null){
      throw new NoTreball();
    }
    return t;
  }

  public void imprimir(int pr) throws PDIdreta, NoImpressora, NoUser, NoTreballUser, CuaBuida {
    Treball t;
    
    // Existeix la impressora?
    if (pr>=MAX_IMPR){
      throw new NoImpressora();
    }
    // La segent crida en retornara null si encara queden copies per fer o
    // el treball si ja han sigut realitzades totes les seves copies
    t = printer[pr].imprimir();

    if (t != null){
      // Busquem l'usuari propietari del treball
      users.principi();
      while ((!users.finalLlista()) && (!((Usuari)(users.actual())).id().equals(t.usuari()))){
        users.avanzar();
      }
    
      if (((Usuari)(users.actual())).id().equals(t.usuari())) { // El trobem
        ((Usuari)(users.actual())).treball_finalitzat(t);  // Ha finalitzat el treball l'ha de treure de la seva llista
      } else {
        throw new NoUser();
      }
    }
  }

  public Llista llistar_treballs_usuari(String userid) throws PDIdreta, NoUser {
    // Busquem usuari
    users.principi();
    while ((!users.finalLlista()) && (!((Usuari)(users.actual())).id().equals(userid))){
      users.avanzar();
    }
    if (((Usuari)(users.actual())).id().equals(userid)) { // Trobat
      return ((Usuari)(users.actual())).llistat();
    } else {
      throw new NoUser();
    }
  }

  public int num_treballs_pendents_usuari(String userid) throws PDIdreta, NoUser {
    // Busquem usuari
    users.principi();
    while ((!users.finalLlista()) && (!((Usuari)(users.actual())).id().equals(userid))){
      users.avanzar();
    }
    if (((Usuari)(users.actual())).id().equals(userid)) { // Trobat
      return ((Usuari)(users.actual())).treballs();
    } else {
      throw new NoUser();
    }
  }

}
